#include "assert.h"

#include "stdio.h"
#include "math.h"
#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_rotozoom.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "collision.h"
#include "samples.h"
#include "animscript.h"
#include "conditionscript.h"
#include "actionscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "interfaces.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activemap.h"
#include "activemog2.h"
#include "activeinterface.h"

#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int surface_flags;
extern int n_collisions;

ActiveOBJECT::ActiveOBJECT(PLACED_OBJECT &o,ActiveROOM *r,ActiveINTERFACE *pinterf)
{
	int i;
	int j;

	ID=o.ID;
	LVK_ID=-1;
	o.object->get_link(o.object->default_tile,0,0,&i,&j);
	old_x=x=o.x+i;	
	old_y=y=o.y+j;
	precision_x=0;
	precision_y=0;
	horizontal_circularity_count=0;
	vertical_circularity_count=0;
	object=o.object;
	room=r;
	interf=pinterf;
	damage=o.object->damage;
	object_plane=-1;
	g_state_finished=false;

	state=o.object->init_state;
	old_state=o.object->init_state;
	state_timmer=0;
	energy=o.object->init_energy;
	object_variables.Copy(o.object->object_variables);
	invincible_script=0;
	jumping_ctnt=o.object->jumping_ctnt;
	gravity_ctnt=o.object->gravity_ctnt;
	walking_speed=o.object->walking_speed;
	ladder_up_speed=o.object->ladder_up_speed;
	ladder_down_speed=o.object->ladder_down_speed;

	left_key=SDLK_UNKNOWN;
	right_key=SDLK_UNKNOWN;
	up_key=SDLK_UNKNOWN;
	down_key=SDLK_UNKNOWN;
	fire1_key=SDLK_UNKNOWN;
	fire2_key=SDLK_UNKNOWN;
	fire3_key=SDLK_UNKNOWN;
	fire4_key=SDLK_UNKNOWN;

	/* Drawing_mode: */ 
	dm_drawing_mode=object->dm_drawing_mode;
	dm_factor=object->dm_factor;
	dm_color1_r=object->dm_color1_r;
	dm_color1_g=object->dm_color1_g;
	dm_color1_b=object->dm_color1_b;
	dm_color1_a=object->dm_color1_a;
	dm_color2_r=object->dm_color2_r;
	dm_color2_g=object->dm_color2_g;
	dm_color2_b=object->dm_color2_b;
	dm_color2_a=object->dm_color2_a;
	dm_blink_f0=0;
	dm_blink_f1=0;
	dm_blink_speed=0;

	scale=object->scale;
	angle=object->angle;

	terminate=false;

	state_object_foreign=new ActiveOBJECT *[object->n_states+object->n_s_states+1];
	for(j=0;j<object->n_states+object->n_s_states+1;j++) state_object_foreign[j]=0;

	for(j=0;j<object->nparts;j++) g_state_anim[j]=new AnimSCRIPT *[object->n_states];
	g_state_action=new ActionSCRIPT *[object->n_states];
	for(i=0;i<object->n_states;i++) {
		for(j=0;j<object->nparts;j++) g_state_anim[j][i]=new AnimSCRIPT(*object->g_state_anim[j][i]);
		for(j=0;j<object->nparts;j++) g_state_anim[j][i]->init();
		
		g_state_action[i]=new ActionSCRIPT(*object->g_state_action[i]);
		g_state_action[i]->init();
	} /* for */ 

	check_collision=false;
	for(j=0;j<object->nparts;j++) {
		tile[j]=o.object->default_tile[j];
		old_tile[j]=o.object->default_tile[j];
	} /* for */ 
	if (o.object->control==CTRL_STANDARD && o.object->n_states>0 && state<o.object->n_states) {
		for(j=0;j<object->nparts;j++) {
			old_tile[j]=g_state_anim[j][state]->get_tile();
			change_tile(g_state_anim[j][state]->get_tile(),j);
		} /* for */ 
	} /* if */ 

	for(j=0;j<object->nparts;j++) g_s_state_anim[j]=new AnimSCRIPT *[object->n_s_states];
	g_s_state_action=new ActionSCRIPT *[object->n_s_states];
	g_s_state_condition=new ConditionSCRIPT *[object->n_s_states];
	state_before_special=0;
	g_s_state_executing=new bool[object->n_s_states];
	for(i=0;i<object->n_s_states;i++) {
		for(j=0;j<object->nparts;j++) g_s_state_anim[j][i]=new AnimSCRIPT(*object->g_s_state_anim[j][i]);
		for(j=0;j<object->nparts;j++) g_s_state_anim[j][i]->init();
		
		g_s_state_action[i]=new ActionSCRIPT(*object->g_s_state_action[i]);
		g_s_state_action[i]->init();

		g_s_state_condition[i]=new ConditionSCRIPT(*object->g_s_state_condition[i]);
		g_s_state_condition[i]->init();
		g_s_state_executing[i]=false;
	} /* for */ 


	switch(object->control) {
	case CTRL_CHARACTER: 
						for(i=0;i<CPOS_NSTATES;i++) {
							for(j=0;j<object->nparts;j++) state_anim[j][i]=new AnimSCRIPT(*object->state_anim[j][i]);
							for(j=0;j<object->nparts;j++) state_anim[j][i]->init();
						} /* for */ 

						character_initialization();
					    break;
	default:
			for(i=0;i<CPOS_NSTATES;i++) {
				for(j=0;j<object->nparts;j++) state_anim[j][i]=0;
			} /* for */ 

			break;
	} /* switch */ 

	/* Hierarchy: */ 
	parent=0;
	relative_precedence=0;
	glued_part=0;
	glued_link=0;
	glued_xoffs=0;
	glued_yoffs=0;

	/* Text: */ 
	text_initialization(object);

	gw_dx=object->gw_dx;
	gw_dy=object->gw_dy;
	gw_scale=object->gw_scale;
	gw_min_hs_x=object->gw_min_hs_x;
	gw_min_hs_y=object->gw_min_hs_y;
	gw_max_hs_x=object->gw_max_hs_x;
	gw_max_hs_y=object->gw_max_hs_y;
	gw_last_viewx=gw_last_viewy=0;

	if (object->menu_title!=0) {
		menu_title=new char[strlen(object->menu_title)+1];
		strcpy(menu_title,object->menu_title);
	} else {
		menu_title=0;
	} /* if */ 
	menu_active=0;
	menu_timmer=0;
	menu_selected=0;
	menu_blink_timmer=0;

	/* Make the variable modifications introduced in the PLACED_OBJECT: */ 
	{
		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(o.modified_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			int ival;
			char *sval;
			int type;

			type=v->get(&ival,&sval,0);
			if (type==VALUE_INTEGER) {
				set_variable_value(v->get_name(),ival);
			} else if (type==VALUE_STRING) {
				set_variable_value(v->get_name(),sval);
			} /* if */ 
		} /* while */ 
	}

} /* ActiveOBJECT::ActiveOBJECT */ 


ActiveOBJECT::ActiveOBJECT(int nx,int ny,int pID,OBJECT *o,ActiveROOM *r,ActiveINTERFACE *pinterf)
{
	int i;
	int j;

	ID=pID;
	LVK_ID=-1;
	old_x=x=nx;
	old_y=y=ny;
	precision_x=0;
	precision_y=0;
	horizontal_circularity_count=0;
	vertical_circularity_count=0;
	object=o;
	room=r;
	interf=pinterf;
	damage=o->damage;
	object_plane=-1;
	g_state_finished=false;

	state=o->init_state;
	old_state=o->init_state;
//	special_state=-1;
	old_state=0;
	state_timmer=0;
	energy=o->init_energy;
	object_variables.Copy(o->object_variables);
	invincible_script=0;
	jumping_ctnt=o->jumping_ctnt;
	gravity_ctnt=o->gravity_ctnt;
	walking_speed=o->walking_speed;
	ladder_up_speed=o->ladder_up_speed;
	ladder_down_speed=o->ladder_down_speed;

	left_key=SDLK_UNKNOWN;
	right_key=SDLK_UNKNOWN;
	up_key=SDLK_UNKNOWN;
	down_key=SDLK_UNKNOWN;
	fire1_key=SDLK_UNKNOWN;
	fire2_key=SDLK_UNKNOWN;
	fire3_key=SDLK_UNKNOWN;
	fire4_key=SDLK_UNKNOWN;

	/* Drawing_mode: */ 
	dm_drawing_mode=object->dm_drawing_mode;
	dm_factor=object->dm_factor;
	dm_color1_r=object->dm_color1_r;
	dm_color1_g=object->dm_color1_g;
	dm_color1_b=object->dm_color1_b;
	dm_color1_a=object->dm_color1_a;
	dm_color2_r=object->dm_color2_r;
	dm_color2_g=object->dm_color2_g;
	dm_color2_b=object->dm_color2_b;
	dm_color2_a=object->dm_color2_a;
	dm_blink_f0=0;
	dm_blink_f1=0;
	dm_blink_speed=0;

	scale=object->scale;
	angle=object->angle;

	terminate=false;

	state_object_foreign=new ActiveOBJECT *[object->n_states+object->n_s_states+1];
	for(j=0;j<object->n_states+object->n_s_states+1;j++) state_object_foreign[j]=0;

	for(j=0;j<object->nparts;j++) g_state_anim[j]=new AnimSCRIPT *[object->n_states];
	g_state_action=new ActionSCRIPT *[object->n_states];
	for(i=0;i<object->n_states;i++) {
		for(j=0;j<object->nparts;j++) g_state_anim[j][i]=new AnimSCRIPT(*object->g_state_anim[j][i]);
		for(j=0;j<object->nparts;j++) g_state_anim[j][i]->init();
		
		g_state_action[i]=new ActionSCRIPT(*object->g_state_action[i]);
		g_state_action[i]->init();
	} /* for */ 

	check_collision=false;
	for(j=0;j<o->nparts;j++) {
		tile[j]=o->default_tile[j];
		old_tile[j]=o->default_tile[j];
	} /* for */ 
	if (o->control==CTRL_STANDARD && o->n_states>0 && state<o->n_states) {
		for(j=0;j<object->nparts;j++) {
			old_tile[j]=g_state_anim[j][state]->get_tile();			
			change_tile(g_state_anim[j][state]->get_tile(),j);
		} /* for */ 
	} /* if */ 

	for(j=0;j<object->nparts;j++) g_s_state_anim[j]=new AnimSCRIPT *[object->n_s_states];
	g_s_state_action=new ActionSCRIPT *[object->n_s_states];
	g_s_state_condition=new ConditionSCRIPT *[object->n_s_states];
	state_before_special=0;
	g_s_state_executing=new bool[object->n_s_states];
	for(i=0;i<object->n_s_states;i++) {
		for(j=0;j<object->nparts;j++) g_s_state_anim[j][i]=new AnimSCRIPT(*object->g_s_state_anim[j][i]);
		for(j=0;j<object->nparts;j++) g_s_state_anim[j][i]->init();
		
		g_s_state_action[i]=new ActionSCRIPT(*object->g_s_state_action[i]);
		g_s_state_action[i]->init();

		g_s_state_condition[i]=new ConditionSCRIPT(*object->g_s_state_condition[i]);
		g_s_state_condition[i]->init();
		g_s_state_executing[i]=false;
	} /* for */ 

	switch(object->control) {
	case CTRL_CHARACTER: 
						for(i=0;i<CPOS_NSTATES;i++) {
							for(j=0;j<object->nparts;j++) state_anim[j][i]=new AnimSCRIPT(*object->state_anim[j][i]);
							for(j=0;j<object->nparts;j++) state_anim[j][i]->init();
						} /* for */ 

						character_initialization();
					    break;
	default:
			for(i=0;i<CPOS_NSTATES;i++) {
				for(j=0;j<object->nparts;j++) state_anim[j][i]=0;
			} /* for */ 

			break;
	} /* switch */ 


	/* Hierarchy: */ 
	parent=0;
	relative_precedence=0;
	glued_part=0;
	glued_link=0;
	glued_xoffs=0;
	glued_yoffs=0;

	/* Text: */ 
	text_initialization(object);

	gw_dx=object->gw_dx;
	gw_dy=object->gw_dy;
	gw_scale=object->gw_scale;
	gw_min_hs_x=object->gw_min_hs_x;
	gw_min_hs_y=object->gw_min_hs_y;
	gw_max_hs_x=object->gw_max_hs_x;
	gw_max_hs_y=object->gw_max_hs_y;
	gw_last_viewx=gw_last_viewy=0;

	if (object->menu_title!=0) {
		menu_title=new char[strlen(object->menu_title)+1];
		strcpy(menu_title,object->menu_title);
	} else {
		menu_title=0;
	} /* if */ 
	menu_active=0;
	menu_timmer=0;
	menu_selected=0;
	menu_blink_timmer=0;

} /* ActiveOBJECT::ActiveOBJECT */ 


ActiveOBJECT::~ActiveOBJECT(void)
{
	int i;
	int j;
	ActiveOBJECT *o;

	delete invincible_script;
	invincible_script=0;

	while(!who_pushed_me.EmptyP()) who_pushed_me.ExtractIni();
	while(!objects_created.EmptyP()) objects_created.ExtractIni();

	object_variables.Delete();

	other_instances.Rewind();
	while(!other_instances.EmptyP()) {
		o=other_instances.ExtractIni();
		o->other_instances.DeleteElement(this);
	} /* while */ 

	for(i=0;i<CPOS_NSTATES;i++) {
		for(j=0;j<object->nparts;j++) delete state_anim[j][i];
		for(j=0;j<object->nparts;j++) state_anim[j][i]=0;
	} /* for */ 

	for(j=0;j<object->n_states+object->n_s_states+1;j++) state_object_foreign[j]=0;
	delete []state_object_foreign;
	state_object_foreign=0;

	for(i=0;i<object->n_states;i++) {
		for(j=0;j<object->nparts;j++) delete g_state_anim[j][i];
		for(j=0;j<object->nparts;j++) g_state_anim[j][i]=0;
		delete g_state_action[i];
		g_state_action[i]=0;
	} /* for */ 
	for(j=0;j<object->nparts;j++) delete []g_state_anim[j];
	for(j=0;j<object->nparts;j++) g_state_anim[j]=0;
	delete []g_state_action;

	for(i=0;i<object->n_s_states;i++) {
		for(j=0;j<object->nparts;j++) delete g_s_state_anim[j][i];
		for(j=0;j<object->nparts;j++) g_s_state_anim[j][i]=0;
		delete g_s_state_action[i];
		g_s_state_action[i]=0;
		delete g_s_state_condition[i];
		g_s_state_condition[i]=0;
	} /* for */ 
	delete []g_s_state_action;
	delete []g_s_state_condition;
	for(j=0;j<object->nparts;j++) delete []g_s_state_anim[j];
	for(j=0;j<object->nparts;j++) g_s_state_anim[j]=0;
	delete []g_s_state_executing;

	/* Hierarchy: */ 
	parent=0;
	relative_precedence=0;
	glued_part=0;
	glued_link=0;
	glued_xoffs=0;
	glued_yoffs=0;

	/* Text: */ 
	if (text_text!=0) delete []text_text;
	if (text_font!=0) delete []text_font;
	text_text=0;
	text_font=0;

	while(!gw_focusing.EmptyP()) gw_focusing.ExtractIni();

	delete []menu_title;

} /* ActiveOBJECT::~ActiveOBJECT */ 

/*
bool ActiveOBJECT::save(FILE *fp,MOG2 *game)
{
	fprintf(fp,"%i %i %i\n",x,y,game->objects.SearchObjRef(object));

	return true;
} /* ActiveOBJECT::save */ 

/*
bool ActiveOBJECT::load(FILE *fp,MOG2 *game)
{
	int tmp;

	if (3!=fscanf(fp,"%i %i %i",&x,&y,&tmp)) return false;

	object=game->objects[tmp];

	return true;
} /* ActiveOBJECT::load */ 

 
int ActiveOBJECT::get_dx(void)
{
	return object->get_dx(tile);
} /* ActiveOBJECT::get_dx */ 


int ActiveOBJECT::get_dy(void)
{
	return object->get_dy(tile);
} /* ActiveOBJECT::get_dy */ 


bool drawing_order(ActiveOBJECT *o1,ActiveOBJECT *o2)
{
	if (o1->relative_precedence+o1->p_offset_o>=o2->relative_precedence+o2->p_offset_o) return true;
	return false;
} /* drawing_order */ 


void ActiveOBJECT::draw(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp)
{
	List<ActiveOBJECT> l,l2;
	ActiveOBJECT *o,*o2,*root;

	if (object->control==CTRL_TEXT) {
		/* Draw a Text object: */ 
		text_draw(sx,sy,sfc,vp);
	} else if (object->control==CTRL_GAMEWINDOW) {
		/* Draw a Game Window: */ 
		gw_draw(sx,sy,sfc,vp);
	} else if (object->control==CTRL_MENU) {
		/* Draw a Menu: */ 
		menu_draw(sx,sy,sfc,vp);
	} else {

		/* Sort the children: */ 
		independent_children.Sort(&drawing_order);
		glued_children.Sort(&drawing_order);

		/* Draw the parts in order: */ 
		root=this;
		l.Instance(independent_children);
		l2.Instance(glued_children);
		l.Rewind();
		l2.Rewind();
		if (l.EndP()) o=0;
				 else o=l.GetObj();
		if (l2.EndP()) o2=0;
				  else o2=l2.GetObj();
		while(!l.EndP() || !l2.EndP() || root!=0) {
			if (root!=0 &&
				(o==0 ||
				 o->relative_precedence<root->relative_precedence) &&
				(o2==0 ||
				o2->relative_precedence<root->relative_precedence)) {
				if (absolute_scale()==1.0F && absolute_angle()==0.0F) {
					if (dm_drawing_mode==4) {
						int kh=0,kv=0,oh=0,ov=0,wave_height=0;
						int distance=0;
						root->get_variable_value("refraction_distance",&distance,0,0);
						root->get_variable_value("refraction_kh",&kh,0,0);
						root->get_variable_value("refraction_kv",&kv,0,0);
						root->get_variable_value("refraction_oh",&oh,0,0);
						root->get_variable_value("refraction_ov",&ov,0,0);
						root->get_variable_value("refraction_wave_height",&wave_height,0,0);
						root->object->draw_refraction(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,distance,kh,kv,oh,ov,wave_height,dm_factor);
					} else if (dm_drawing_mode==3) {
						int kh=0,kv=0,oh=0,ov=0,wave_height=0;
						int center_x=sfc->w/2;
						int center_y=sfc->h/2;
						root->get_variable_value("reflection_kh",&kh,0,0);
						root->get_variable_value("reflection_kv",&kv,0,0);
						root->get_variable_value("reflection_oh",&oh,0,0);
						root->get_variable_value("reflection_ov",&ov,0,0);
						root->get_variable_value("reflection_wave_height",&wave_height,0,0);
						root->get_variable_value("reflection_center_x",&center_x,0,0);
						root->get_variable_value("reflection_center_y",&center_y,0,0);
						root->object->draw_reflection(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,kh,kv,oh,ov,wave_height,center_x,center_y,dm_factor);
					} else if (dm_drawing_mode==2) root->object->draw_bicolor(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,dm_factor,dm_color1_r,dm_color1_g,dm_color1_b,dm_color1_a,dm_color2_r,dm_color2_g,dm_color2_b,dm_color2_a);
					else if (dm_drawing_mode==1) root->object->draw_shaded(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,dm_factor,dm_color1_r,dm_color1_g,dm_color1_b,dm_color1_a);
					else root->object->draw(sx+x,sy+y,tile,root->p_offset_p,sfc,vp);
				} else {
					if (dm_drawing_mode==4) {
						int kh=0,kv=0,oh=0,ov=0,wave_height=0;
						int distance=0;
						root->get_variable_value("refraction_distance",&distance,0,0);
						root->get_variable_value("refraction_kh",&kh,0,0);
						root->get_variable_value("refraction_kv",&kv,0,0);
						root->get_variable_value("refraction_oh",&oh,0,0);
						root->get_variable_value("refraction_ov",&ov,0,0);
						root->get_variable_value("refraction_wave_height",&wave_height,0,0);
						root->object->draw_refraction_transform(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,absolute_scale(),absolute_angle(),distance,kh,kv,oh,ov,wave_height,dm_factor);
					} else if (dm_drawing_mode==3) {
						int kh=0,kv=0,oh=0,ov=0,wave_height=0;
						int center_x=sfc->w/2;
						int center_y=sfc->h/2;
						root->get_variable_value("reflection_kh",&kh,0,0);
						root->get_variable_value("reflection_kv",&kv,0,0);
						root->get_variable_value("reflection_oh",&oh,0,0);
						root->get_variable_value("reflection_ov",&ov,0,0);
						root->get_variable_value("reflection_wave_height",&wave_height,0,0);
						root->get_variable_value("reflection_center_x",&center_x,0,0);
						root->get_variable_value("reflection_center_y",&center_y,0,0);
						root->object->draw_reflection_transform(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,absolute_scale(),absolute_angle(),kh,kv,oh,ov,wave_height,center_x,center_y,dm_factor);
					} else if (dm_drawing_mode==2) root->object->draw_bicolor_transform(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,absolute_scale(),absolute_angle(),dm_factor,dm_color1_r,dm_color1_g,dm_color1_b,dm_color1_a,dm_color2_r,dm_color2_g,dm_color2_b,dm_color2_a);
					else if (dm_drawing_mode==1) root->object->draw_shaded_transform(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,absolute_scale(),absolute_angle(),dm_factor,dm_color1_r,dm_color1_g,dm_color1_b,dm_color1_a);
					else root->object->draw_transform(sx+x,sy+y,tile,root->p_offset_p,sfc,vp,absolute_scale(),absolute_angle());
				} /* if */ 
				root=0;
			} else {
				if (o2!=0 &&
					(o==0 ||
					 o->relative_precedence<o2->relative_precedence)) {
					if (o2->room==room) o2->draw(sx,sy,sfc,vp);
					l2.Next();
					if (l2.EndP()) o2=0;
							  else o2=l2.GetObj();
				} else {
					if (o->room==room) o->draw(sx,sy,sfc,vp);
					l.Next();
					if (l.EndP()) o=0;
							 else o=l.GetObj();
				} /* if */ 
			} /* if */ 
		} /* while */ 
	} /* if */ 

} /* ActiveOBJECT::draw */ 


void ActiveOBJECT::draw_collision_mask(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp,int constitution)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;

	if (object!=0 &&
		((object->constitution&constitution)!=0)) {
		if (absolute_scale()==1.0F && absolute_angle()==0.0F) object->draw_collision_mask(sx+x,sy+y,tile,sfc,vp);
														 else object->draw_collision_mask_transform(sx+x,sy+y,tile,sfc,vp,absolute_scale(),absolute_angle());
	} /* if */ 

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		o->draw_collision_mask(sx,sy,sfc,vp,constitution);
	} /* while */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		o->draw_collision_mask(sx,sy,sfc,vp,constitution);
	} /* while */ 

} /* ActiveOBJECT::draw_collision_mask */ 



void ActiveOBJECT::precycle(void)
{
	int j;

	old_x=x;
	old_y=y;
	old_scale=absolute_scale();
	old_angle=absolute_angle();
	for(j=0;j<object->nparts;j++) old_tile[j]=tile[j];

	/* Clear the PUSHING and the CREATED lists: */ 
	while(!who_pushed_me.EmptyP()) who_pushed_me.ExtractIni();
	while(!objects_created.EmptyP()) objects_created.ExtractIni();


	/* Children objects: */ 
	{
		List<ActiveOBJECT> l;
		ActiveOBJECT *o;

		l.Instance(independent_children);
		l.Rewind();
		while(l.Iterate(o)) o->precycle();

		l.Instance(glued_children);
		l.Rewind();
		while(l.Iterate(o)) o->precycle();
	}

} /* ActiveOBJECT::precycle */ 


bool ActiveOBJECT::cycle(KEYBOARDSTATE **LVK,int n_LVKs)
{
	int j,before_collision_x,before_collision_y;
	bool retval=true;
	int desired_tile[MAX_PARTS];

	for(j=0;j<object->nparts;j++) desired_tile[j]=tile[j];

//	if (room!=0 && !room->room->horizontal_circularity) horizontal_circularity_count=0;
//	if (room!=0 && !room->room->vertical_circularity) vertical_circularity_count=0;
	
	/* If multiple instances of the same object are present in several rooms,			*/ 
	/* only the instance whose hot_spot is inside the room (here should be only one)	*/ 
	/* has to be actualized, the other will be copies:									*/ 
	if (!other_instances.EmptyP()) {
		int hx,hy;
		hotspot_coordinates(&hx,&hy);
		if (hx<0 || hx>=room->room->dx*TILESIZE ||
			hy<0 || hy>=room->room->dy*TILESIZE) return retval;
	} /* if */ 


	/* Children objects: */ 
	{
		List<ActiveOBJECT> l;
		ActiveOBJECT *o;

		l.Instance(independent_children);
		l.Rewind();
		while(l.Iterate(o)) o->cycle(LVK,n_LVKs);

		l.Instance(glued_children);
		l.Rewind();
		while(l.Iterate(o)) o->cycle(LVK,n_LVKs);
	}

	/* Blinking: */ 
	if (dm_blink_speed!=0) {
		dm_factor+=dm_blink_speed;
		if (dm_blink_speed>0) {
			if (dm_factor>=dm_blink_f1) {
				dm_factor=dm_blink_f1;
				dm_blink_speed=-dm_blink_speed;
			} /* if */ 
		} else {
			if (dm_factor<=dm_blink_f0) {
				dm_factor=dm_blink_f0;
				dm_blink_speed=-dm_blink_speed;
			} /* if */ 
		} /* if */ 
	} /* if */ 

	/* Special states: */ 
	{
		int max_priority=-1,tmp_s;
		int i,t,j;
		bool found=false;

		tmp_s=highest_priority_special();
		if (tmp_s!=-1) max_priority=object->g_s_state_priority[tmp_s];

		for(i=0;i<object->n_s_states;i++) {
			if (!g_s_state_executing[i] &&
				object->g_s_state_priority[i]>max_priority && 
				g_s_state_condition[i]->cycle(this,room,get_map(),get_game(),interf,get_game()->app,&(state_object_foreign[object->n_states+i]),LOCATION_OBJECT,LVK,n_LVKs)) {
				found=true;
				/* Entering in a SPECIAL state: */ 
				if (object->g_s_state_exclusiveness[i]) {
					if (tmp_s==-1) state_before_special=state;
					g_s_state_executing[i]=true;
					if (object->control==CTRL_CHARACTER) state=CPOS_NSTATES+i;
													else state=object->n_states+i;
				} else {
					g_s_state_executing[i]=true;
				} /* if */ 

				for(j=0;j<object->nparts;j++) {
					g_s_state_anim[j][i]->init();
					t=g_s_state_anim[j][i]->get_tile();
					p_offset_p[j]=g_s_state_anim[j][i]->get_p_offset_p();
					if (g_s_state_anim[j][i]->get_sample()!=0) {
						PLAYSAMPLE *ps;

						ps=new PLAYSAMPLE();
						ps->sample=g_s_state_anim[j][i]->get_sample();
						ps->volume=(g_s_state_anim[j][i]->get_sample_volume()*127)/100;
						ps->pan=0;
						ps->room=room;
						ps->placed=false;
						if (room!=0) {
							ps->placed=true;
							ps->x=x;
							ps->y=y;
						} /* if */ 
						get_game()->add_sample(ps);
					} /* if */ 
					desired_tile[j]=t;
				} /* for */ 
				p_offset_o=g_s_state_anim[0][i]->get_p_offset_o();
				g_s_state_action[i]->init();
			} /* if */ 
		} /* for */ 
	}

	/* Object cycles: */ 
	check_collision=object->collision_flag;
	switch(object->control) {
	case CTRL_STANDARD:  
	case CTRL_TEXT:
	case CTRL_GAMEWINDOW:
	case CTRL_MENU:
		/* Animations: */ 
		if (object->n_states>0 && state<object->n_states) {
			int t;

			state_change=false;
			for(j=0;j<object->nparts;j++) {
				g_state_anim[j][state]->cycle();
				t=g_state_anim[j][state]->get_tile();
				p_offset_p[j]=g_state_anim[j][state]->get_p_offset_p();
				desired_tile[j]=t;
				if (g_state_anim[j][state]->get_sample()!=0) {
					PLAYSAMPLE *ps;

					ps=new PLAYSAMPLE();
					ps->sample=g_state_anim[j][state]->get_sample();
					ps->volume=(g_state_anim[j][state]->get_sample_volume()*127)/100;
					ps->pan=0;
					ps->room=room;
					ps->placed=false;
					if (room!=0) {
						ps->placed=true;
						ps->x=x;
						ps->y=y;
					} /* if */ 
					get_game()->add_sample(ps);
				} /* if */ 
			} /* for */ 
			p_offset_o=g_state_anim[0][state]->get_p_offset_o();

			/* Action: */ 
			if (!g_state_finished &&
				g_state_action[state]->cycle(this,room,get_map(),get_game(),interf,get_game()->app,&(state_object_foreign[state]),LOCATION_OBJECT,LVK,n_LVKs)) {
				g_state_finished=true;
			} /* if */ 

			if (state_change) {
				for(j=0;j<object->nparts;j++) {
					g_state_anim[j][state]->init();
					t=g_state_anim[j][state]->get_tile();
					p_offset_p[j]=g_state_anim[j][state]->get_p_offset_p();
					if (g_state_anim[j][state]->get_sample()!=0) {
						PLAYSAMPLE *ps;

						ps=new PLAYSAMPLE();
						ps->sample=g_state_anim[j][state]->get_sample();
						ps->volume=(g_state_anim[j][state]->get_sample_volume()*127)/100;
						ps->pan=0;
						ps->room=room;
						ps->placed=false;
						if (room!=0) {
							ps->placed=true;
							ps->x=x;
							ps->y=y;
						} /* if */ 
						get_game()->add_sample(ps);
					} /* if */ 
					desired_tile[j]=t;
				} /* for */ 
				p_offset_o=g_state_anim[0][state]->get_p_offset_o();

				g_state_action[state]->init();
				g_state_finished=false;
			} /* if */ 

			state_timmer++;
		} /* if */ 

		retval=true;
		if (object->control==CTRL_MENU) retval=menu_cycle(LVK,n_LVKs);
		break;

	case CTRL_CHARACTER: 
		retval=character_cycle((int *)desired_tile,LVK,n_LVKs);
		break;
	} /* switch */ 

	/* Special states: */ 
	{
		int t,j,i;
		int tmp_s,tmp_ps;
		int max_priority[MAX_PARTS];
		int old_special_state=highest_priority_special();
		state_change=false;

		/* Animations: */ 
		for(j=0;j<object->nparts;j++) max_priority[j]=-1;
		for(i=0;i<object->n_s_states;i++) {
			if (g_s_state_executing[i] &&
				!object->g_s_state_exclusiveness[i]) {
				for(j=0;j<object->nparts;j++) {
					g_s_state_anim[j][i]->cycle();
					t=g_s_state_anim[j][i]->get_tile();
					p_offset_p[j]=g_s_state_anim[j][i]->get_p_offset_p();
					if (t!=ANIM_INVALID &&
						max_priority[j]<object->g_s_state_priority[i]) {
						max_priority[j]=object->g_s_state_priority[i];
						desired_tile[j]=t;
					} /* if */ 
					if (g_s_state_anim[j][i]->get_sample()!=0) {
						PLAYSAMPLE *ps;

						ps=new PLAYSAMPLE();
						ps->sample=g_s_state_anim[j][i]->get_sample();
						ps->volume=(g_s_state_anim[j][i]->get_sample_volume()*127)/100;
						ps->pan=0;
						ps->room=room;
						ps->placed=false;
						if (room!=0) {
							ps->placed=true;
							ps->x=x;
							ps->y=y;
						} /* if */ 
						get_game()->add_sample(ps);
					} /* if */ 
				} /* for */ 
			} /* if */ 
		} /* for */ 
		tmp_s=highest_priority_special();
		if (tmp_s!=-1) {
			for(j=0;j<object->nparts;j++) {
				g_s_state_anim[j][tmp_s]->cycle();
				t=g_s_state_anim[j][tmp_s]->get_tile();
				p_offset_p[j]=g_s_state_anim[j][tmp_s]->get_p_offset_p();
				if (t!=ANIM_INVALID &&
					max_priority[j]<object->g_s_state_priority[tmp_s]) {
					max_priority[j]=object->g_s_state_priority[tmp_s];
					desired_tile[j]=t;
				} /* if */ 
				if (g_s_state_anim[j][tmp_s]->get_sample()!=0) {
					PLAYSAMPLE *ps;

					ps=new PLAYSAMPLE();
					ps->sample=g_s_state_anim[j][tmp_s]->get_sample();
					ps->volume=(g_s_state_anim[j][tmp_s]->get_sample_volume()*127)/100;
					ps->pan=0;
					ps->room=room;
					ps->placed=false;
					if (room!=0) {
						ps->placed=true;
						ps->x=x;
						ps->y=y;
					} /* if */ 
					get_game()->add_sample(ps);
				} /* if */ 
			} /* for */ 
			p_offset_o=g_s_state_anim[0][tmp_s]->get_p_offset_o();
		} /* if */ 

		/* Action: */ 
		tmp_s=highest_priority_special();
		tmp_ps=-1;
		if (tmp_s!=-1) {
			tmp_ps=object->g_s_state_priority[tmp_s];
			executed_state_change=false;
			if (g_s_state_action[tmp_s]->cycle(this,room,get_map(),get_game(),interf,get_game()->app,&(state_object_foreign[object->n_states+tmp_s]),LOCATION_OBJECT,LVK,n_LVKs) ||
				executed_state_change) {
				g_s_state_executing[tmp_s]=false;
				if (highest_priority_special()==-1) state=state_before_special;
											   else state=highest_priority_special()+CPOS_NSTATES;
				if (executed_state_change) state_change=true;
			} /* if */ 
		} /* if */ 	
		for(i=0;i<object->n_s_states;i++) {
			executed_state_change=false;
			if (g_s_state_executing[i] &&
				!object->g_s_state_exclusiveness[i]) {
				if (g_s_state_action[i]->cycle(this,room,get_map(),get_game(),interf,get_game()->app,&(state_object_foreign[object->n_states+i]),LOCATION_OBJECT,LVK,n_LVKs)) {
					g_s_state_executing[i]=false;
				} /* if */ 
				if (executed_state_change) state_change=true;
			} /* if */ 
		} /* for */ 

		if (tmp_s!=-1 &&
			highest_priority_special()==-1 &&
			state_change && 
			object->control!=CTRL_CHARACTER) {
			for(j=0;j<object->nparts;j++) {
				g_state_anim[j][state]->init();
				t=g_state_anim[j][state]->get_tile();
				p_offset_p[j]=g_state_anim[j][state]->get_p_offset_p();
				desired_tile[j]=t;
				if (g_state_anim[j][state]->get_sample()!=0) {
					PLAYSAMPLE *ps;

					ps=new PLAYSAMPLE();
					ps->sample=g_state_anim[j][state]->get_sample();
					ps->volume=(g_state_anim[j][state]->get_sample_volume()*127)/100;
					ps->pan=0;
					ps->room=room;
					ps->placed=false;
					if (room!=0) {
						ps->placed=true;
						ps->x=x;
						ps->y=y;
					} /* if */ 
					get_game()->add_sample(ps);
				} /* if */ 
			} /* for */ 
			p_offset_o=g_state_anim[0][state]->get_p_offset_o();

			g_state_action[state]->init();
		} /* if */ 	

	}

	for(j=0;j<object->nparts;j++) change_tile(desired_tile[j],j);

	recompute_glued_position();

	before_collision_x=x;
	before_collision_y=y;

	/* Check for collisions: */ 
	if (check_collision && !glued()) {
		switch(object->control) {
		case CTRL_STANDARD:
			if (old_x!=x || old_y!=y ||
				old_scale!=absolute_scale() ||
				old_angle!=absolute_angle()) {
				/* If a solid object moves, all the pushable objects must move acordingly: */ 
				if (object->constitution==CONS_SOLID ||
					object->constitution==CONS_SOLID_SURFACE) {

					int i;
					List<ActiveOBJECT> l;
					ActiveOBJECT *o;
					List<ROOM> rl;
					ROOM *r;
					List<ActiveROOM> l2;
					ActiveROOM *ar;
					bool found;

					{
						int ax1,ay1,ax2,ay2,borderx,bordery;
						get_absbbox(&ax1,&ay1,&ax2,&ay2);

						borderx=old_x-x;
						if (borderx<0) borderx=-borderx;
						bordery=old_y-y;
						if (bordery<0) bordery=-bordery;
						borderx+=2;
						bordery+=2;

						room->map->find_rooms_for_object(ax1-borderx,ay1-bordery,(ax2-ax1)+borderx*2,(ay2-ay1)+bordery*2,&rl);
					}

					/* Before making the collision tests, all the other instances must be updated: */ 
					update_instances();

					while(!rl.EmptyP()) {
						r=rl.ExtractIni();

						l2.Instance(room->map->active_rooms);
						l2.Rewind();
						found=false;
						while(!found && l2.Iterate(ar)) {
							if (ar->room==r) {	
								found=true;
								for(i=0;i<MAX_OBJECT_PLANES;i++) {
									l.Instance(ar->objects[i]);
									l.Rewind();
									while(l.Iterate(o)) {
										if (has_to_be_pushed(o)) {
											o->set_push_object(this);
											/* Move the object o: */ 
											int xd=x-old_x;
											int yd=y-old_y;
											int rx,ry;

											rx=o->room->room->get_x()*TILESIZE;
											ry=o->room->room->get_y()*TILESIZE;

											if (!room->map->object_collision(o,o->x+xd+rx,o->y+ry,true)) {
												o->x+=xd;
												o->update_instances();	
												o->reinit_created_objects(xd,0);
											} /* if */ 
											if (!room->map->object_collision(o,o->x+rx,o->y+yd+ry,true)) {
												o->y+=yd;
												o->update_instances();	
												o->reinit_created_objects(0,yd);
											} else {
												room->map->object_collision(o,o->x+rx,o->y+yd+ry,true);
											} /* if */ 
										} /* if */ 
									} /* while */ 
								} /* for */ 

							} /* if */ 
						} /* while */ 
					} /* while */ 
				
				} /* if */ 

				standard_collision();

			} /* if */ 
			break;
		case CTRL_CHARACTER: 
			character_collision();
			break;
		case CTRL_TEXT:
			break;
		case CTRL_GAMEWINDOW:
			break;
		case CTRL_MENU:
			break;
		} /* switch */ 
	} /* if */ 

	/* Check for collisions: */ 
	switch(object->control) {
	case CTRL_STANDARD:
		break;
	case CTRL_CHARACTER: 
		character_postcycle();
		break;
	case CTRL_TEXT:
		break;
	case CTRL_GAMEWINDOW:
		break;
	} /* switch */ 

	recompute_glued_position();

	/* Actualize the other instances:	*/ 
	if (x!=before_collision_x || y!=before_collision_y) {
		reinit_created_objects(x-before_collision_x,y-before_collision_y);
	} /* if */ 		
	

	update_instances();

	/* ... */ 
//	if (fp==0) fp=fopen("report.txt","w+");
//	if (strcmp(object->name,"popolon")==0) fprintf(fp,"%i(after) - popolon(%p): %i %i (%i)\n",game->get_current_cycle(),this,x,y,other_instances.Length());
//	if (strcmp(object->name,"vertical lever (multiroom)")==0) fprintf(fp,"%i(after) - vertical lever (multiroom)(%p): %i %i\n",game->get_current_cycle(),this,x,y);
	/* ... */ 

	return retval;
} /* ActiveOBJECT::cycle */ 




void ActiveOBJECT::standard_collision(void)
{
	int rx,ry;
	int candidate_x=old_x,candidate_y=old_y;
	int init_col=0;
	bool g_found=false;

	rx=room->room->get_x()*TILESIZE;
	ry=room->room->get_y()*TILESIZE;

	/* Check for collisions: */ 
	if (old_x!=x || old_y!=y ||
		old_scale!=absolute_scale() ||
		old_angle!=absolute_angle()) {

		if (room->map->object_collision(this,true)!=0) {
			bool found=false;
			int i,maxy=old_y,maxx=old_x;

			init_col=room->map->object_collision(this,old_x+rx,old_y+ry,true)&CONS_SOLID_SURFACE;
			if (x!=old_x) {
				found=false;
				if (x<old_x) {
					for(i=old_x;i>=x;i--) {
						if ((room->map->object_collision(this,i+rx,old_y+ry,true)&CONS_SOLID)==0) {
							found=true;
							maxx=i;
						} /* if */ 
					} /* if */ 
				} else {
					for(i=old_x;i<=x;i++) {
						if ((room->map->object_collision(this,i+rx,old_y+ry,true)&CONS_SOLID)==0) {
							found=true;
							maxx=i;
						} /* if */ 
					} /* if */ 
				} /* if */ 

				if (found) {
					g_found=true;
					candidate_x=maxx;
				} /* if */ 
			} /* if */ 

			if (y!=old_y) {
				found=false;
				i=old_y;
				if (y<old_y) {
					for(i=old_y;i>=y;i--) {
						if ((room->map->object_collision(this,candidate_x+rx,i+ry,true)&CONS_SOLID)==0) {
							maxy=i;
							found=true;
						} /* if */ 
					} /* for */ 
				} else {
					for(i=old_y;i<=y;i++) {
						if ((room->map->object_collision(this,candidate_x+rx,i+ry,true)|init_col)==init_col) {
							maxy=i;
							found=true;
						} /* if */  
					} /* for */ 
				} /* if */ 
				if (found) {
					g_found=true;
					candidate_y=maxy;
				} else {
					g_found=false;
				} /* if */ 
			} /* if */ 
		} /* if */ 

		if ((room->map->object_collision(this,candidate_x+rx,candidate_y+ry,true)&CONS_SOLID)!=0 ||
			!g_found) {

			find_without_collision_place(8+abs(x-old_x),8+abs(y-old_y));
			if ((room->map->object_collision(this,x+rx,y+ry,true)&CONS_SOLID)!=0) {
				x=old_x;
				y=old_y;
			} /* if */ 
		} else {
			x=candidate_x;
			y=candidate_y;
		} /* if */ 
	} /* if */ 

} /* ActiveOBJECT::standard_collision */ 



void ActiveOBJECT::change_tile(int newtile,int part)
{
	if (newtile==ANIM_INVALID) return;

	if (newtile!=tile[part]) {
		tile[part]=newtile;

		/* Check for collisions: */ 
		if (check_collision) {
			find_without_collision_place(8,8);
		} /* if */ 
	} /* if */ 
} /* ActiveOBJECT::change_tile */ 


bool ActiveOBJECT::passive(void) 
{
	return object->passive();
} /* ActiveOBJECT::pasive */ 


bool ActiveOBJECT::glued(void)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;

	if (parent==0) return false;

	l.Instance(parent->glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o==this) return true;
	} /* while */ 

	return false;
} /* ActiveOBJECT::glued(void) */ 



bool ActiveOBJECT::cmp(ActiveOBJECT *obj)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;

	if (obj==this) return true;

	l.Instance(other_instances);
	l.Rewind();
	while(l.Iterate(o)) {
		if (obj==o) return true;
	} /* while */ 

	return false;
} /* ActiveOBJECT::cmp */ 


void ActiveOBJECT::absolute_hotspot_coordinates(int *rx,int *ry)
{
	*rx=x+room->room->x*TILESIZE;
	*ry=y+room->room->y*TILESIZE;
} /* ActiveOBJECT::absolute_hotspot_coordinates */ 


void ActiveOBJECT::hotspot_coordinates(int *rx,int *ry)
{
	*rx=x;
	*ry=y;
} /* ActiveOBJECT::hotspot_coordinates */ 


bool ActiveOBJECT::planep(int i)
{
	if (room->objects[i].MemberRefP(this)) {
		object_plane=i;
		return true;
	} /* if */ 

	return false;
} /* ActiveOBJECT::planep */ 


int ActiveOBJECT::get_object_plane(void)
{
	int i;

	if (object_plane!=-1) return object_plane;
	if (parent!=0) return parent->get_object_plane();

	for(i=0;i<MAX_OBJECT_PLANES;i++) if (planep(i)) return i;

	object_plane=-1;
	return object_plane;
} /* get_object_plane */ 


void ActiveOBJECT::copy_state(ActiveOBJECT *o)
{
	int j,i;

	if (!multiroom()) room=o->room;

	if (o->room==room &&
		room->room->horizontal_circularity) {
		x=(o->x+o->room->room->x*TILESIZE)-(room->room->x*TILESIZE)+
		  (o->horizontal_circularity_count-horizontal_circularity_count)*room->room->get_dx()*TILESIZE;
	} else {
		x=(o->x+o->room->room->x*TILESIZE)-(room->room->x*TILESIZE);
	} /* if */ 

	if (o->room==room &&
		room->room->vertical_circularity) {
		y=(o->y+o->room->room->y*TILESIZE)-(room->room->y*TILESIZE)+
		  (o->vertical_circularity_count-vertical_circularity_count)*room->room->get_dy()*TILESIZE;;
	} else {
		y=(o->y+o->room->room->y*TILESIZE)-(room->room->y*TILESIZE);
	} /* if */ 

	LVK_ID=o->LVK_ID;
	object=o->object;
	scale=o->scale;
	angle=o->angle;
	relative_precedence=o->relative_precedence;
	state=o->state;
	old_state=o->old_state;
	state_timmer=o->state_timmer;
	continuous_state_change=o->continuous_state_change;
	terminate=o->terminate;

	for(j=0;j<object->nparts;j++) tile[j]=o->tile[j];
	if (object_plane!=-1 && o->object_plane!=-1 &&
		object_plane!=o->object_plane) {
		object_plane=o->object_plane;
		room->objects_to_change_plane.Add(this);
	} /* if */ 
	/* Copy the values of the variables: */ 
	if (o->object_variables.Length()!=object_variables.Length()) {
		object_variables.Delete();
		object_variables.Copy(o->object_variables);
	} else {
		List<VARIABLE> l,l2;
		VARIABLE *v,*v2;

		l.Instance(object_variables);
		l2.Instance(o->object_variables);
		l.Rewind();
		l2.Rewind();
		while(l.Iterate(v) &&
			l2.Iterate(v2)) {
			v->instance(v2);
		} /* while */ 
	} /* if */ 

	energy=o->energy;
	jumping_inertia=o->jumping_inertia;
	if (invincible_script==0) {
		if (o->invincible_script!=0) {
			invincible_script=new ActionSCRIPT();
			invincible_script->copy_state(o->invincible_script);
		} /* if */ 
	} else {
		if (o->invincible_script==0) {
			delete invincible_script;
			invincible_script=0;
		} else {
			invincible_script->copy_state(o->invincible_script);
		} /* if */ 
	} /* if */ 
	damage=o->damage;
	jumping_ctnt=o->jumping_ctnt;
	gravity_ctnt=o->gravity_ctnt;
	walking_speed=o->walking_speed;
	ladder_up_speed=o->ladder_up_speed;
	ladder_down_speed=o->ladder_down_speed;

	left_key=o->left_key;
	right_key=o->right_key;
	up_key=o->up_key;
	down_key=o->down_key;
	fire1_key=o->fire1_key;
	fire2_key=o->fire2_key;
	fire3_key=o->fire3_key;
	fire4_key=o->fire4_key;
        
    dm_drawing_mode=o->dm_drawing_mode;
	dm_factor=o->dm_factor;
	dm_color1_r=o->dm_color1_r;
	dm_color1_g=o->dm_color1_g;
	dm_color1_b=o->dm_color1_b;
	dm_color1_a=o->dm_color1_a;
	dm_color2_r=o->dm_color2_r;
	dm_color2_g=o->dm_color2_g;
	dm_color2_b=o->dm_color2_b;
	dm_color2_a=o->dm_color2_a;
	dm_blink_f0=o->dm_blink_f0;
	dm_blink_f1=o->dm_blink_f1;
	dm_blink_speed=o->dm_blink_speed;

	/* Copy also the action and animation scripts: */ 
	switch(object->control) {
	case CTRL_STANDARD:

		if (object->n_states>0 && state>=0 && state<object->n_states) {
			for(j=0;j<object->nparts;j++) g_state_anim[j][state]->copy_state(o->g_state_anim[j][state]);
			g_state_action[state]->copy_state(o->g_state_action[state]);
		} /* if */ 

		for(j=0;j<object->n_s_states;j++) {
			g_s_state_executing[j]=o->g_s_state_executing[j];
			if (g_s_state_executing[j]) {
				g_s_state_condition[j]->copy_state(o->g_s_state_condition[j]);
				for(i=0;i<object->nparts;i++) g_s_state_anim[i][j]->copy_state(o->g_s_state_anim[i][j]);
				g_s_state_action[j]->copy_state(o->g_s_state_action[j]);
			} /* if */ 
		} /* for */ 
		break;
	case CTRL_CHARACTER: 

		if (state<CPOS_NSTATES) {
			for(j=0;j<object->nparts;j++) state_anim[j][state]->copy_state(o->state_anim[j][state]);
		} /* if */ 

		for(j=0;j<object->n_s_states;j++) {
			g_s_state_executing[j]=o->g_s_state_executing[j];
			if (g_s_state_executing[j]) {
				g_s_state_condition[j]->copy_state(o->g_s_state_condition[j]);
				for(i=0;i<object->nparts;i++) g_s_state_anim[i][j]->copy_state(o->g_s_state_anim[i][j]);
				g_s_state_action[j]->copy_state(o->g_s_state_action[j]);
			} /* if */ 
		} /* for */ 
		break;
	case CTRL_TEXT:
		break;
	case CTRL_GAMEWINDOW:
		break;
	} /* switch */ 

	state_before_special=o->state_before_special;

	/* Hierarchy: */ 
	{
		List<ActiveOBJECT> l,l2,todelete,toadd;
		ActiveOBJECT *no,*co,*coo;
		bool found;
		int i;

		for(i=0;i<2;i++) {
			/* Update all the children objects: */ 
			if (i==0) {
				l.Instance(glued_children);
				l2.Instance(o->glued_children);
			} else {
				l.Instance(independent_children);
				l2.Instance(o->independent_children);
			} /* if */ 
			l2.Rewind();
			while(l2.Iterate(coo)) {
				found=false;
				l.Rewind();
				while(!found && l.Iterate(co)) {
					if (coo->cmp(co)) {
						co->copy_state(coo);
						found=true;
					} /* if */ 
				} /* while */ 

				if (!found) {
					int absx,absy;
					/* Create a new object: */ 
					coo->absolute_hotspot_coordinates(&absx,&absy);
					no=new ActiveOBJECT(absx-room->room->x*TILESIZE,absy-room->room->y*TILESIZE,coo->ID,coo->object,room,interf);
					no->is_instance(coo);
					no->copy_state(coo);
					toadd.Add(no);
				} /* if */ 

			} /* while */ 
			if (i==0) {
				while(!toadd.EmptyP()) glued_children.Add(toadd.ExtractIni());
			} else {
				while(!toadd.EmptyP()) independent_children.Add(toadd.ExtractIni());
			} /* if */ 

			/* Delete all the objects that do not exist on 'o': */ 
			l.Rewind();
			while(l.Iterate(co)) {
				found=false;
				l2.Rewind();
				while(!found && l2.Iterate(coo)) {
					if (coo->cmp(co)) found=true;
				} /* while */ 

				if (!found) {
					todelete.Add(co);
				} /* if */ 
			} /* while */ 
			if (i==0) {
				while(!todelete.EmptyP()) {
					no=todelete.ExtractIni();
					glued_children.DeleteElement(no);
					delete no;
				} /* while */ 
			} else {
				while(!todelete.EmptyP()) {
					no=todelete.ExtractIni();
					independent_children.DeleteElement(no);
					delete no;
				} /* while */ 
			} /* if */ 
		} /* for */ 
	}

	/* Text: */ 
	if (object->control==CTRL_TEXT) text_initialization(o);
	if (object->control==CTRL_GAMEWINDOW) {
		gw_dx=o->gw_dx;
		gw_dy=o->gw_dy;
		gw_scale=o->gw_scale;
		gw_focusing=o->gw_focusing;
		gw_min_hs_x=o->gw_min_hs_x;
		gw_min_hs_y=o->gw_min_hs_y;
		gw_max_hs_x=o->gw_max_hs_x;
		gw_max_hs_y=o->gw_max_hs_y;
	} /* if */ 

	if (object->control==CTRL_MENU) {
		if (o->menu_title!=0) {
			menu_title=new char[strlen(o->menu_title)+1];
			strcpy(menu_title,o->menu_title);
		} else {
			menu_title=0;
		} /* if */ 
		menu_active=o->menu_active;
		menu_timmer=o->menu_timmer;
		menu_selected=o->menu_selected;
	} /* if */ 
	
} /* ActiveOBJECT::copy_state */ 


void ActiveOBJECT::is_instance(ActiveOBJECT *o)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o2;

	l.Instance(o->other_instances);
	l.Rewind();
	while(l.Iterate(o2)) {
		other_instances.Add(o2);
		o2->other_instances.Add(this);
	} /* while */ 

	other_instances.Add(o);
	o->other_instances.Add(this);

} /* ActiveOBJECT::is_instance */ 


void ActiveOBJECT::precision_move(int x_move,int y_move)
{
	precision_x+=x_move;
	precision_y+=y_move;
	if (precision_x>0) {
		x+=precision_x/100;
		precision_x=precision_x%100;
	} /* if */ 
	if (precision_x<0) {
		x-=(-precision_x)/100;
		precision_x=-((-precision_x)%100);
	} /* if */ 
	if (precision_y>0) {
		y+=precision_y/100;
		precision_y=precision_y%100;
	} /* if */ 
	if (precision_y<0) {
		y-=(-precision_y)/100;
		precision_y=-((-precision_y)%100);
	} /* if */ 
} /* ActiveOBJECT::precision_move */ 


void ActiveOBJECT::find_without_collision_place(int xtolerance,int ytolerance)
{
	if (room->map->object_collision(this,true)) {
		int i,j;
		int new_x=x;
		int new_y=y;
		bool found=false;
		int ax,ay;
		absolute_hotspot_coordinates(&ax,&ay);
		
		for(found=false,i=1;!found && i<xtolerance+ytolerance;i++) {
			for(j=i;!found && j>=0;j--) {
				if (abs(j)<xtolerance && abs(i-j)<ytolerance) {
					if ((room->map->object_collision(this,ax+j,ay+i-j,true)&CONS_SOLID)==0) {
						new_x=x+j;
						new_y=y+(i-j);
						found=true;
					} /* if */ 
					if (!found && (room->map->object_collision(this,ax+j,ay-(i-j),true)&CONS_SOLID)==0) {
						new_x=x+j;
						new_y=y-(i-j);
						found=true;
					} /* if */ 
					if (!found && (room->map->object_collision(this,ax-j,ay+i-j,true)&CONS_SOLID)==0) {
						new_x=x-j;
						new_y=y+(i-j);
						found=true;
					} /* if */ 
					if (!found && (room->map->object_collision(this,ax-j,ay-(i-j),true)&CONS_SOLID)==0) {
						new_x=x-j;
						new_y=y-(i-j);
						found=true;
					} /* if */ 
				} /* if */ 
			} /* for */ 
		} /* for */ 

		x=new_x;
		y=new_y;
	} /* if */ 
} /* ActiveOBJECT::find_wihout_collision_place */ 


bool ActiveOBJECT::collision(ActiveOBJECT *o,bool hierarchical)
{
	return collision(x,y,o,hierarchical);
} /* ActiveOBJECT::collision */ 


bool ActiveOBJECT::collision(ActiveOBJECT *o,int constitution,int o_constitution,bool hierarchical)
{
	return collision(x,y,o,constitution,o_constitution,hierarchical);
} /* ActiveOBJECT::collision */ 


bool ActiveOBJECT::collision(int x,int y,ActiveOBJECT *o,bool hierarchical)
{
	return collision(x,y,o,CONS_ALL,CONS_ALL,hierarchical);
} /* ActiveOBJECT::collision */ 


bool ActiveOBJECT::collision(int x,int y,ActiveOBJECT *o,int constitution,int o_constitution,bool hierarchical)
{
	int i;
	int oax,oay;
	int rx=room->room->x*TILESIZE;
	int ry=room->room->y*TILESIZE;
	float as=o->absolute_scale(),aa=o->absolute_angle();

	n_collisions++;

	if ((o->object->constitution&o_constitution)!=0) {
		o->absolute_hotspot_coordinates(&oax,&oay);

		for(i=0;i<o->object->nparts;i++) {
			if (o->tile[i]>=0 && o->tile[i]<o->object->ntiles &&
				o->tile[0]>=0 && o->tile[0]<o->object->ntiles) {
				int cx=0,cy=0;
				if (as!=1 || aa!=0) o->object->get_part_position_transform(i,o->tile,as,aa,&cx,&cy);
							   else o->object->get_part_position(i,o->tile,&cx,&cy);
				cx+=oax;
				cy+=oay;

				if (low_level_collision(rx+x,ry+y,cx,cy,o->object->tiles[o->tile[i]]->get_collision_data(as,aa),constitution,hierarchical)) {
					return true;
				} /* if */ 
			} /* if */ 
		} /* for */ 
	} /* if */ 

	/* Children: */ 
	{
		List<ActiveOBJECT> l;
		ActiveOBJECT *child;

		l.Instance(o->independent_children);
		l.Rewind();
		while(l.Iterate(child)) {
			if (child!=this &&
				collision(x,y,child,constitution,o_constitution,hierarchical)) return true;
		} /* while */ 

		l.Instance(o->glued_children);
		l.Rewind();
		while(l.Iterate(child)) {
			if (child!=this &&
				collision(x,y,child,constitution,o_constitution,hierarchical)) return true;
		} /* while */ 
	}

	return false;
} /* ActiveOBJECT::collision */ 


/* The collision are performed using ABSOLUTE COORDINATES. This way, you can make the test	*/ 
/* for objects that are in the same room, and objects that aren't.							*/ 
bool ActiveOBJECT::low_level_collision(int ax,int ay,int ax2,int ay2,MOG2CollisionData *o_cdata,int constitution,bool hierarchical)
{
	int i;
	float as=absolute_scale(),aa=absolute_angle();

	if ((object->constitution&constitution)!=0) {
		for(i=0;i<object->nparts;i++) {
			if (tile[i]>=0 && tile[i]<object->ntiles) {
				int cx=0,cy=0;
				if (as!=1 || aa!=0) object->get_part_position_transform(i,tile,as,aa,&cx,&cy);
							   else object->get_part_position(i,tile,&cx,&cy);
				cx+=ax;
				cy+=ay;
				if (MOG2_collision_check(object->tiles[tile[i]]->get_collision_data(as,aa),cx,cy,
						 		 o_cdata,ax2,ay2)) {
					return true;
				} /* if */ 
			} /* if */ 
		} /* for */ 
	} /* if */ 

	/* Children: */ 
	if (hierarchical) {
		List<ActiveOBJECT> l;
		ActiveOBJECT *o;

		l.Instance(independent_children);
		l.Rewind();
		while(l.Iterate(o)) {
			int cx=0,cy=0;
			o->absolute_hotspot_coordinates(&cx,&cy);
			if (o->low_level_collision(cx,cy,ax2,ay2,o_cdata,constitution,true)) return true;
		} /* while */ 

		l.Instance(glued_children);
		l.Rewind();
		while(l.Iterate(o)) if (o->low_level_collision(ax+(o->old_x-old_x),ay+(o->old_y-old_y),ax2,ay2,o_cdata,constitution,true)) return true;
	} /* if */ 

	return false;
} /* ActiveOBJECT::low_level_collision */ 


ActiveOBJECT *ActiveOBJECT::collision2(int x,int y,ActiveOBJECT *o,int constitution,int o_constitution,ConditionSCRIPT *s,ActiveOBJECT **foreign_object,bool hierarchical)
{
	int i;
	int oax,oay;
	int rx;
	int ry;
	int cx=0,cy=0;
	float as=o->absolute_scale(),aa=o->absolute_angle();

	n_collisions++;

	if ((o->object->constitution&o_constitution)!=0) {
		rx=room->room->x*TILESIZE;
		ry=room->room->y*TILESIZE;
//		oax=o->old_x+o->room->room->x*TILESIZE;
//		oay=o->old_y+o->room->room->y*TILESIZE;
		oax=o->x+o->room->room->x*TILESIZE;
		oay=o->y+o->room->room->y*TILESIZE;

		for(i=0;i<o->object->nparts;i++) {
			if (o->old_tile[i]>=0 && o->old_tile[i]<o->object->ntiles &&
				o->old_tile[0]>=0 && o->old_tile[0]<o->object->ntiles) {
				if (as!=1 || aa!=0) o->object->get_part_position_transform(i,o->tile,as,aa,&cx,&cy);
							   else o->object->get_part_position(i,o->tile,&cx,&cy);
				cx+=oax;
				cy+=oay;

				if (low_level_collision2(rx+x,ry+y,cx,cy,o->object->tiles[o->old_tile[i]]->get_collision_data(as,aa),constitution,hierarchical)) {
					*foreign_object=o;
					if (s->cycle(this,this->room,get_map(),get_game(),interf,get_game()->app,foreign_object,LOCATION_OBJECT,0,0)) {
						return o;
					} else {
						*foreign_object=0;
					} /* if */ 
				} /* if */ 
			} /* if */ 
		} /* for */ 
	} /* if */ 

	/* Children: */ 
	{
		List<ActiveOBJECT> l;
		ActiveOBJECT *child,*tmp;

		l.Instance(o->independent_children);
		l.Rewind();
		while(l.Iterate(child)) {
			if (child!=this) {
				tmp=collision2(x,y,child,constitution,o_constitution,s,foreign_object,hierarchical);
				if (tmp!=0) return tmp;
			} /* if */ 
		} /* while */ 

		l.Instance(o->glued_children);
		l.Rewind();
		while(l.Iterate(child)) {
			if (child!=this) {
				tmp=collision2(x,y,child,constitution,o_constitution,s,foreign_object,hierarchical);
				if (tmp!=0) return tmp;
			} /* if */ 
		} /* while */ 
	}

	return 0;
} /* ActiveOBJECT::collision2 */ 


/* The collision are performed using ABSOLUTE COORDINATES. This way, you can make the test	*/ 
/* for objects that are in the same room, and objects that aren't.							*/ 
bool ActiveOBJECT::low_level_collision2(int ax,int ay,int ax2,int ay2,MOG2CollisionData *o_cdata,int constitution,bool hierarchical)
{
	int i;
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;
	float as=absolute_scale(),aa=absolute_angle();

	if ((object->constitution&constitution)!=0) {
		for(i=0;i<object->nparts;i++) {
			if (old_tile[i]>=0 && old_tile[i]<object->ntiles) {
				int cx=0,cy=0;
				if (as!=1 || aa!=0) object->get_part_position_transform(i,tile,as,aa,&cx,&cy);
							   else object->get_part_position(i,tile,&cx,&cy);
				cx+=ax;
				cy+=ay;
				if (MOG2_collision_check(object->tiles[old_tile[i]]->get_collision_data(as,aa),cx,cy,
								 o_cdata,ax2,ay2)) {
					return true;
				} /* if */ 
			} /* if */ 
		} /* for */ 
	} /* if */ 

	if (hierarchical) {
		/* Children: */ 
		l.Instance(independent_children);
		l.Rewind();
		while(l.Iterate(o)) {
			int cx=0,cy=0;
//			cx=o->old_x+o->room->room->x*TILESIZE;
//			cy=o->old_y+o->room->room->y*TILESIZE;
			cx=o->x+o->room->room->x*TILESIZE;
			cy=o->y+o->room->room->y*TILESIZE;
			if (o->low_level_collision(cx,cy,ax2,ay2,o_cdata,constitution,true)) return true;
		} /* while */ 

		l.Instance(glued_children);
		l.Rewind();
		while(l.Iterate(o)) if (o->low_level_collision(ax+(o->old_x-old_x),ay+(o->old_y-old_y),ax2,ay2,o_cdata,constitution,true)) return true;
	} /* if */ 

	return false;
} /* ActiveOBJECT::low_level_collision2 */ 


bool ActiveOBJECT::has_to_be_pushed(ActiveOBJECT *o)
{
	if (o->object->pushable_flag) {

		/* Test if this object has been already pushed: */ 
		if (o->who_pushed_me.MemberRefP(this)) return false;
		{
			List<ActiveOBJECT> l;
			ActiveOBJECT *oi;
			
			l.Instance(other_instances);
			l.Rewind();
			while(l.Iterate(oi)) {
				if (o->who_pushed_me.MemberRefP(oi)) return false;
			} /* while */ 
		}

		if (old_y==y) {
			/* Only X move: */ 
			if (collision(x-2,y-2,o,CONS_SOLID|CONS_SOLID_SURFACE,o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE,true) ||
				collision(x+2,y-2,o,CONS_SOLID|CONS_SOLID_SURFACE,o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE,true)) {
				return true;
			} /* if */ 
		} else {
			/* Also Y move: */ 
			if (y>old_y) {
				/* moving down: */ 
				if (collision(x,old_y-2,o,CONS_SOLID|CONS_SOLID_SURFACE,o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE,true) ||
					collision(x,y+2,o,CONS_SOLID|CONS_SOLID_SURFACE,o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE,true)) {
					return true;
				} /* if */ 
			} else {
				/* moving up: */ 
				if (collision(x,y-2,o,CONS_SOLID|CONS_SOLID_SURFACE,o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE,true)) {
					return true;
				} /* if */ 
			} /* if */ 
		} /* if */ 
	} /* if */ 
	return false;
} /* ActiveOBJECT::has_to_be_pushed */ 


void ActiveOBJECT::update_instances(void) 
{
	if (!other_instances.EmptyP()) {
		List<ActiveOBJECT> l;
		ActiveOBJECT *o;
		
		l.Instance(other_instances);
		l.Rewind();
		while(l.Iterate(o)) {
			if (o!=this) o->copy_state(this);
		} /* while */ 
	} /* if */ 
} /* ActiveOBJECT::update_instances */ 


void ActiveOBJECT::set_push_object(ActiveOBJECT *o) 
{
	who_pushed_me.Add(o);

	if (!other_instances.EmptyP()) {
		List<ActiveOBJECT> l;
		ActiveOBJECT *oi;
		
		l.Instance(other_instances);
		l.Rewind();
		while(l.Iterate(oi)) {
			oi->who_pushed_me.Add(o);
		} /* while */ 
	} /* if */ 
} /* ActiveOBJECT::set_push_object */ 


void ActiveOBJECT::get_bbox(int *x1,int *y1,int *x2,int *y2)
{
	object->get_bbox(x,y,tile,x1,y1,x2,y2);
} /* ActiveOBJECT::get_bbox */ 


void ActiveOBJECT::get_absbbox(int *ax1,int *ay1,int *ax2,int *ay2)
{
	object->get_bbox(x,y,tile,ax1,ay1,ax2,ay2);
	*ax1+=room->room->x*TILESIZE;
	*ay1+=room->room->y*TILESIZE;
	*ax2+=room->room->x*TILESIZE;
	*ay2+=room->room->y*TILESIZE;
} /* ActiveOBJECT::get_bbox */ 


void ActiveOBJECT::get_family_bbox(int *x1,int *y1,int *x2,int *y2)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;
	int tx1,ty1,tx2,ty2;

	object->get_bbox(x,y,tile,x1,y1,x2,y2);

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		o->get_family_bbox(&tx1,&ty1,&tx2,&ty2);
		if (tx1<*x1) *x1=tx1;
		if (tx2>*x2) *x2=tx2;
		if (ty1<*y1) *y1=ty1;
		if (ty2>*y2) *y2=ty2;
	} /* while */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		o->get_family_bbox(&tx1,&ty1,&tx2,&ty2);
		if (tx1<*x1) *x1=tx1;
		if (tx2>*x2) *x2=tx2;
		if (ty1<*y1) *y1=ty1;
		if (ty2>*y2) *y2=ty2;
	} /* while */ 
} /* ActiveOBJECT::get_family_bbox */ 


void ActiveOBJECT::get_family_multiroom_bbox(int *x1,int *y1,int *x2,int *y2)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;
	int tx1,ty1,tx2,ty2;

	object->get_bbox(x,y,tile,x1,y1,x2,y2);

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->multiroom()) {
			o->get_family_bbox(&tx1,&ty1,&tx2,&ty2);
			if (tx1<*x1) *x1=tx1;
			if (tx2>*x2) *x2=tx2;
			if (ty1<*y1) *y1=ty1;
			if (ty2>*y2) *y2=ty2;
		} /* if */ 
	} /* while */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->multiroom()) {
			o->get_family_bbox(&tx1,&ty1,&tx2,&ty2);
			if (tx1<*x1) *x1=tx1;
			if (tx2>*x2) *x2=tx2;
			if (ty1<*y1) *y1=ty1;
			if (ty2>*y2) *y2=ty2;
		} /* if */ 
	} /* while */ 
} /* ActiveOBJECT::get_family_multiroom_bbox */ 


void ActiveOBJECT::get_family_absbbox(int *ax1,int *ay1,int *ax2,int *ay2)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;
	int tx1,ty1,tx2,ty2;

	object->get_bbox(x,y,tile,ax1,ay1,ax2,ay2);
	*ax1+=room->room->x*TILESIZE;
	*ay1+=room->room->y*TILESIZE;
	*ax2+=room->room->x*TILESIZE;
	*ay2+=room->room->y*TILESIZE;

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		o->get_family_absbbox(&tx1,&ty1,&tx2,&ty2);
		if (tx1<*ax1) *ax1=tx1;
		if (tx2>*ax2) *ax2=tx2;
		if (ty1<*ay1) *ay1=ty1;
		if (ty2>*ay2) *ay2=ty2;
	} /* while */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		o->get_family_absbbox(&tx1,&ty1,&tx2,&ty2);
		if (tx1<*ax1) *ax1=tx1;
		if (tx2>*ax2) *ax2=tx2;
		if (ty1<*ay1) *ay1=ty1;
		if (ty2>*ay2) *ay2=ty2;
	} /* while */ 
} /* ActiveOBJECT::get_family_absbbox */ 


void ActiveOBJECT::get_family_multiroom_absbbox(int *ax1,int *ay1,int *ax2,int *ay2)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;
	int tx1,ty1,tx2,ty2;

	object->get_bbox(x,y,tile,ax1,ay1,ax2,ay2);
	*ax1+=room->room->x*TILESIZE;
	*ay1+=room->room->y*TILESIZE;
	*ax2+=room->room->x*TILESIZE;
	*ay2+=room->room->y*TILESIZE;

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->multiroom()) {
			o->get_family_absbbox(&tx1,&ty1,&tx2,&ty2);
			if (tx1<*ax1) *ax1=tx1;
			if (tx2>*ax2) *ax2=tx2;
			if (ty1<*ay1) *ay1=ty1;
			if (ty2>*ay2) *ay2=ty2;
		} /* if */ 
	} /* while */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->multiroom()) {
			o->get_family_absbbox(&tx1,&ty1,&tx2,&ty2);
			if (tx1<*ax1) *ax1=tx1;
			if (tx2>*ax2) *ax2=tx2;
			if (ty1<*ay1) *ay1=ty1;
			if (ty2>*ay2) *ay2=ty2;
		} /* if */ 
	} /* while */ 
} /* ActiveOBJECT::get_family_multiroom_absbbox */ 


void ActiveOBJECT::reinit_created_objects(int xd,int yd)
{
	int i;
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;
	bool found=false;
	List<ActiveOBJECT> l2;
	ActiveOBJECT *tmp;

	l.Instance(objects_created);
	l.Rewind();
	while(l.Iterate(o)) {
		/* Only those objects not GLUED to the main object have to be moved, since the GLUED
		   objects are moved automatically.														*/ 
		found=false;	

		l2.Instance(glued_children);
		l2.Rewind();
		while(!found && l2.Iterate(tmp)) {
			if (tmp==o) found=true;
		} /* while */ 

		if (!found) {
			o->x+=xd;
			o->y+=yd;
		} /* if */ 

		for(i=0;i<o->object->n_states;i++) {	
			o->g_state_action[i]->init();
		} /* for */ 
		for(i=0;i<o->object->n_s_states;i++) {	
			o->g_s_state_action[i]->init();
			o->g_s_state_condition[i]->init();
		} /* for */ 
		o->update_instances();
	} /* while */ 
} /* ActiveOBJECT::reinit_created_objects */ 


bool ActiveOBJECT::multiroom(void)
{
	if (object->multiroom_flag) return true;
//	if (parent!=0) return parent->multiroom();
	return false;
} /* ActiveOBJECT::multiroom */ 


void ActiveOBJECT::recompute_glued_position(void)
{
	int px,py;
	int lx,ly;

	if (glued()) {
		parent->object->get_part_position(glued_part,parent->tile,&px,&py);
		parent->object->get_link(parent->tile,glued_part,glued_link,&lx,&ly);
		rotate_coordinates(int((px+lx)*parent->absolute_scale()),int((py+ly)*parent->absolute_scale()),parent->absolute_angle(),&lx,&ly);
		x=long(parent->x+lx);
		y=long(parent->y+ly);
	} /* if */ 

	/* Children objects: */ 
	{
		List<ActiveOBJECT> l;
		ActiveOBJECT *o;

		l.Instance(glued_children);
		l.Rewind();
		while(l.Iterate(o)) o->recompute_glued_position();
	}
} /* ActiveOBJECT::recompute_glued_position */ 



int ActiveOBJECT::get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval)
{
	if (strcmp(vname,"state")==0) {
		*ival=state;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"this")==0) {
		*oval=this;
		return VALUE_OBJECT;
	} else if (strcmp(vname,"x")==0) {
		*ival=x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"y")==0) {
		*ival=y;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"scale")==0) {
		*ival=scale;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"angle")==0) {
		*ival=angle;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"energy")==0) {
		*ival=energy;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"experience")==0) {
		*ival=object->experience;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"damage")==0) {
		*ival=damage;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"damage_type")==0) {
		*sval=object->damage_type;
		return VALUE_STRING;
	} else if (strcmp(vname,"relative_precedence")==0) {
		*ival=relative_precedence;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"jumping_ctnt")==0) {
		*ival=jumping_ctnt;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gravity_ctnt")==0) {
		*ival=gravity_ctnt;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"walking_speed")==0) {
		*ival=walking_speed;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"ladder_up_speed")==0) {
		*ival=ladder_up_speed;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"ladder_down_speed")==0) {
		*ival=ladder_down_speed;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"dm_drawing_mode")==0) {
		*ival=dm_drawing_mode;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_r")==0) {
		*ival=dm_color1_r;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_g")==0) {
		*ival=dm_color1_g;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_b")==0) {
		*ival=dm_color1_b;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color1_a")==0) {
		*ival=dm_color1_a;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_r")==0) {
		*ival=dm_color2_r;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_g")==0) {
		*ival=dm_color2_g;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_b")==0) {
		*ival=dm_color2_b;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_color2_a")==0) {
		*ival=dm_color2_a;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_factor")==0) {
		*ival=dm_factor;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_blink_f0")==0) {
		*ival=dm_blink_f0;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_blink_f1")==0) {
		*ival=dm_blink_f1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"dm_blink_speed")==0) {
		*ival=dm_blink_speed;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"text_text")==0) {
		*sval=text_text;
		return VALUE_STRING;
	} else if (strcmp(vname,"text_size")==0) {
		*ival=text_size;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_spacing")==0) {
		*ival=text_spacing;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_h_align")==0) {
		*ival=text_h_align;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_v_align")==0) {
		*ival=text_v_align;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_r")==0) {
		*ival=text_r;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_g")==0) {
		*ival=text_g;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_b")==0) {
		*ival=text_b;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_a")==0) {
		*ival=text_a;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame")==0) {
		*ival=text_frame;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_point_x")==0) {
		*ival=text_frame_point_x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_point_y")==0) {
		*ival=text_frame_point_y;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_r1")==0) {
		*ival=text_frame_r1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_g1")==0) {
		*ival=text_frame_g1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_b1")==0) {
		*ival=text_frame_b1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_a1")==0) {
		*ival=text_frame_a1;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_r2")==0) {
		*ival=text_frame_r2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_g2")==0) {
		*ival=text_frame_g2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_b2")==0) {
		*ival=text_frame_b2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_a2")==0) {
		*ival=text_frame_a2;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_width")==0) {
		*ival=text_frame_width;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_frame_radius")==0) {
		*ival=text_frame_radius;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"text_max_w")==0) {
		*ival=text_max_w;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_dx")==0) {
		*ival=gw_dx;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_dy")==0) {
		*ival=gw_dy;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_scale")==0) {
		*ival=gw_scale;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_min_hs_x")==0) {
		*ival=gw_min_hs_x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_min_hs_y")==0) {
		*ival=gw_min_hs_y;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_max_hs_x")==0) {
		*ival=gw_max_hs_x;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"gw_max_hs_y")==0) {
		*ival=gw_max_hs_y;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"key_up")==0) {
		*ival=up_key;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"key_down")==0) {
		*ival=down_key;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"key_left")==0) {
		*ival=left_key;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"key_right")==0) {
		*ival=right_key;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"key_fire1")==0) {
		*ival=fire1_key;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"key_fire2")==0) {
		*ival=fire2_key;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"key_fire3")==0) {
		*ival=fire3_key;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"key_fire4")==0) {
		*ival=fire4_key;
		return VALUE_INTEGER;

	} else if (strcmp(vname,"object_plane")==0) {
		*ival=get_object_plane();
		return VALUE_INTEGER;

	} else if (strcmp(vname,"menu_dx")==0) {
		*ival=object->menu_dx;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_dy")==0) {
		*ival=object->menu_dy;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_title")==0) {
		*sval=menu_title;
		return VALUE_STRING;
	} else if (strcmp(vname,"menu_active")==0) {
		*ival=menu_active;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_selected")==0) {
		*ival=menu_selected;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_appearance_flags")==0) {
		*ival=object->menu_appearance_flags;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_entry_effect_flags")==0) {
		*ival=object->menu_entry_effect_flags;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_blink_timmer")==0) {
		*ival=object->menu_blink_timmer;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_zoom_factor")==0) {
		*ival=object->menu_zoom_factor;
		return VALUE_INTEGER;
	} else if (strcmp(vname,"menu_appear_factor")==0) {
		*ival=object->menu_appear_factor;
		return VALUE_INTEGER;

	} else {
		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(object_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) return v->get(ival,sval,oval);
		} /* while */ 
		return VALUE_NONE;
	} /* if */ 
} /* ActiveOBJECT::get_variable_value */ 


void ActiveOBJECT::set_variable_value(char *vname,int value)
{
	if (strcmp(vname,"state")==0) {
		if ((object->control!=CTRL_CHARACTER && value<object->n_states && value>=0) ||
		    (object->control==CTRL_CHARACTER && value<CPOS_NSTATES && value>=0)) {
			if (object->control==CTRL_CHARACTER) {
				if (value==CPOS_JUMPLEFT) {
					jumping_inertia=-walking_speed;
					state_timmer=0;
				} /* if */ 
				if (value==CPOS_JUMPRIGHT) {
					jumping_inertia=walking_speed;
					state_timmer=0;
				} /* if */ 
			} else {
				g_state_action[value]->init();
			} /* if */ 
			if (highest_priority_special()!=-1) {
				state_before_special=value;	
				g_state_finished=false;
			} else {
				int j;

				state=value;
				state_timmer=0;
				state_change=true;
				g_state_finished=false;
				if (object->control==CTRL_CHARACTER) {
					for(j=0;j<object->nparts;j++) change_tile(state_anim[j][state]->get_tile(),j);
				} else {
					for(j=0;j<object->nparts;j++) change_tile(g_state_anim[j][state]->get_tile(),j);
				} /* if */ 
			} /* if */ 
		} /* if */ 
	} else if (strcmp(vname,"x")==0) {
		x=value;
	} else if (strcmp(vname,"y")==0) {
		y=value;
	} else if (strcmp(vname,"scale")==0) {
		scale=value;
	} else if (strcmp(vname,"angle")==0) {
		angle=value;
		while(angle<0) angle+=360;
		while(angle>=360) angle-=360;
	} else if (strcmp(vname,"energy")==0) {
		energy=value;
	} else if (strcmp(vname,"experience")==0) {
	} else if (strcmp(vname,"damage")==0) {
		damage=value;
	} else if (strcmp(vname,"relative_precedence")==0) {
		relative_precedence=value;
	} else if (strcmp(vname,"jumping_ctnt")==0) {
		jumping_ctnt=value;
	} else if (strcmp(vname,"gravity_ctnt")==0) {
		gravity_ctnt=value;
	} else if (strcmp(vname,"walking_speed")==0) {
		walking_speed=value;
	} else if (strcmp(vname,"ladder_up_speed")==0) {
		ladder_up_speed=value;
	} else if (strcmp(vname,"ladder_down_speed")==0) {
		ladder_down_speed=value;

	} else if (strcmp(vname,"dm_drawing_mode")==0) {
		dm_drawing_mode=value;
	} else if (strcmp(vname,"dm_color1_r")==0) {
		dm_color1_r=value;
	} else if (strcmp(vname,"dm_color1_g")==0) {
		dm_color1_g=value;
	} else if (strcmp(vname,"dm_color1_b")==0) {
		dm_color1_b=value;
	} else if (strcmp(vname,"dm_color1_a")==0) {
		dm_color1_a=value;
	} else if (strcmp(vname,"dm_color2_r")==0) {
		dm_color2_r=value;
	} else if (strcmp(vname,"dm_color2_g")==0) {
		dm_color2_g=value;
	} else if (strcmp(vname,"dm_color2_b")==0) {
		dm_color2_b=value;
	} else if (strcmp(vname,"dm_color2_a")==0) {
		dm_color2_a=value;
	} else if (strcmp(vname,"dm_factor")==0) {
		dm_factor=value;
	} else if (strcmp(vname,"dm_blink_f0")==0) {
		dm_blink_f0=value;
	} else if (strcmp(vname,"dm_blink_f1")==0) {
		dm_blink_f1=value;
	} else if (strcmp(vname,"dm_blink_speed")==0) {
		dm_blink_speed=value;

	} else if (strcmp(vname,"text_size")==0) {
		text_size=value;
	} else if (strcmp(vname,"text_spacing")==0) {
		text_spacing=value;
	} else if (strcmp(vname,"text_h_align")==0) {
		text_h_align=value;
	} else if (strcmp(vname,"text_v_align")==0) {
		text_v_align=value;
	} else if (strcmp(vname,"text_r")==0) {
		text_r=value;
		if (text_r<0) text_r=0;
		if (text_r>255) text_r=255;
	} else if (strcmp(vname,"text_g")==0) {
		text_g=value;
		if (text_g<0) text_g=0;
		if (text_g>255) text_g=255;
	} else if (strcmp(vname,"text_b")==0) {
		text_b=value;
		if (text_b<0) text_b=0;
		if (text_b>255) text_b=255;
	} else if (strcmp(vname,"text_a")==0) {
		text_a=value;
		if (text_a<0) text_a=0;
		if (text_a>255) text_a=255;
	} else if (strcmp(vname,"text_frame")==0) {
		text_frame=value;
	} else if (strcmp(vname,"text_frame_point_x")==0) {
		text_frame_point_x=value;
	} else if (strcmp(vname,"text_frame_point_y")==0) {
		text_frame_point_y=value;
	} else if (strcmp(vname,"text_frame_r1")==0) {
		text_frame_r1=value;
	} else if (strcmp(vname,"text_frame_g1")==0) {
		text_frame_g1=value;
	} else if (strcmp(vname,"text_frame_b1")==0) {
		text_frame_b1=value;
	} else if (strcmp(vname,"text_frame_a1")==0) {
		text_frame_a1=value;
	} else if (strcmp(vname,"text_frame_r2")==0) {
		text_frame_r2=value;
	} else if (strcmp(vname,"text_frame_g2")==0) {
		text_frame_g2=value;
	} else if (strcmp(vname,"text_frame_b2")==0) {
		text_frame_b2=value;
	} else if (strcmp(vname,"text_frame_a2")==0) {
		text_frame_a2=value;
	} else if (strcmp(vname,"text_frame_width")==0) {
		text_frame_width=value;
	} else if (strcmp(vname,"text_frame_radius")==0) {
		text_frame_radius=value;
	} else if (strcmp(vname,"text_max_w")==0) {
		text_max_w=value;
	} else if (strcmp(vname,"object_plane")==0) {
		object_plane=get_object_plane();
		if (object_plane!=value) {
			room->objects_to_change_plane.Add(this);
			if (value<0) value=0;
			if (value>=MAX_OBJECT_PLANES) value=MAX_OBJECT_PLANES-1;
			object_plane=value;
		} /* if */ 

	} else if (strcmp(vname,"gw_dx")==0) {
		gw_dx=value;
	} else if (strcmp(vname,"gw_dy")==0) {
		gw_dy=value;
	} else if (strcmp(vname,"gw_scale")==0) {
		gw_scale=value;
	} else if (strcmp(vname,"gw_min_hs_x")==0) {
		gw_min_hs_x=value;
	} else if (strcmp(vname,"gw_min_hs_y")==0) {
		gw_min_hs_y=value;
	} else if (strcmp(vname,"gw_max_hs_x")==0) {
		gw_max_hs_x=value;
	} else if (strcmp(vname,"gw_max_hs_y")==0) {
		gw_max_hs_y=value;

	} else if (strcmp(vname,"key_up")==0) {
		up_key=value;
	} else if (strcmp(vname,"key_down")==0) {
		down_key=value;
	} else if (strcmp(vname,"key_left")==0) {
		left_key=value;
	} else if (strcmp(vname,"key_right")==0) {
		right_key=value;
	} else if (strcmp(vname,"key_fire1")==0) {
		fire1_key=value;
	} else if (strcmp(vname,"key_fire2")==0) {
		fire2_key=value;
	} else if (strcmp(vname,"key_fire3")==0) {
		fire3_key=value;
	} else if (strcmp(vname,"key_fire4")==0) {
		fire4_key=value;

	} else if (strcmp(vname,"menu_active")==0) {
		menu_active=value;
		menu_timmer=object->menu_appear_factor;
	} else if (strcmp(vname,"menu_selected")==0) {
		menu_selected=value;
	} else {
		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(object_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) {
				v->set(value);
				return;
			} /* if */ 
		} /* while */ 
	} /* if */ 
} /* ActiveOBJECT::set_variable_value */ 


void ActiveOBJECT::set_variable_value(char *vname,char *value)
{
	if (strcmp(vname,"damage_type")==0) {
		/* You cannot set the value of this variable dinamically: */ 
/*
		delete damage_type;
		damage_type=new char[strlen(value)+1];
		strcpy(damage_type,value);
*/
	} else if (strcmp(vname,"dm_drawing_mode")==0) {
		if (strcmp(value,"normal")==0) dm_drawing_mode=0;
		if (strcmp(value,"shaded")==0) dm_drawing_mode=1;
		if (strcmp(value,"bicolor")==0) dm_drawing_mode=2;
		if (strcmp(value,"reflection")==0) dm_drawing_mode=3;
		if (strcmp(value,"refraction")==0) dm_drawing_mode=4;
	} else if (strcmp(vname,"text_text")==0) {
		delete []text_text;
		text_text=new char[strlen(value)+1];
		strcpy(text_text,value);
	} else if (strcmp(vname,"text_font")==0) {
		delete []text_font;
		text_font=new char[strlen(value)+1];
		strcpy(text_font,value);
	} else if (strcmp(vname,"menu_title")==0) {
		delete []menu_title;
		menu_title=new char[strlen(value)+1];
		strcpy(menu_title,value);
	} else {

		List<VARIABLE> l;
		VARIABLE *v;

		l.Instance(object_variables);
		l.Rewind();
		while(l.Iterate(v)) {
			if (v->cmp(vname)) {
				v->set(value);
				return;
			} /* if */ 
		} /* while */ 
	} /* if */ 
} /* ActiveOBJECT::set_variable_value */ 


void ActiveOBJECT::set_variable_value(char *vname,ActiveOBJECT *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(object_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveOBJECT::set_variable_value */ 


bool ActiveOBJECT::offspringp(ActiveOBJECT *object)
{
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;
	
	if (object==this) return true;

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->offspringp(object)) return true;
	} /* while */ 

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->offspringp(object)) return true;
	} /* while */ 

	return false;
} /* ActiveOBJECT::offspringp */ 


int ActiveOBJECT::count_objects(char *name,ConditionSCRIPT *cond,ActiveOBJECT *script_o,ActiveROOM *script_r,ActiveMAP *script_m,ActiveMOG2 *script_g,ActiveINTERFACE *script_i,ActiveMOG2Application *script_a,int script_location)
{
	int count=0;
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;

	if (strcmp(object->name,name)==0) {
		ActiveOBJECT *foreign_object=this;
		if (cond->cycle(script_o,script_r,script_m,script_g,script_i,script_a,&foreign_object,script_location,0,0)) {
			count=1;
		} /* if */ 
	} /* if */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		count+=o->count_objects(name,cond,script_o,script_r,script_m,script_g,script_i,script_a,script_location);
	} /* while */ 

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		count+=o->count_objects(name,cond,script_o,script_r,script_m,script_g,script_i,script_a,script_location);
	} /* while */ 

	return count;
} /* ActiveOBJECT::count_objects */ 


void ActiveOBJECT::delete_terminated_children(void)
{
	List<ActiveOBJECT> l,todelete;
	ActiveOBJECT *o;

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->terminate) {
			todelete.Add(o);
		} else {
			o->delete_terminated_children();
		} /* if */ 
	} /* while */ 

	while(!todelete.EmptyP()) {
		o=todelete.ExtractIni();
		independent_children.DeleteElement(o);
		delete o;
	} /* while */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->terminate) {
			todelete.Add(o);
		} else {
			o->delete_terminated_children();
		} /* if */ 
	} /* while */ 

	while(!todelete.EmptyP()) {
		o=todelete.ExtractIni();
		glued_children.DeleteElement(o);
		delete o;
	} /* while */ 
} /* ActiveOBJECT::delete_terminated_children */ 


void ActiveOBJECT::delete_children_in_room(ActiveROOM *r)
{
	List<ActiveOBJECT> l,todelete;
	ActiveOBJECT *o;

	l.Instance(independent_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->room==r) {
			todelete.Add(o);
		} else {
			o->delete_children_in_room(r);
		} /* if */ 
	} /* while */ 

	while(!todelete.EmptyP()) {
		o=todelete.ExtractIni();
		independent_children.DeleteElement(o);
		delete o;
	} /* while */ 

	l.Instance(glued_children);
	l.Rewind();
	while(l.Iterate(o)) {
		if (o->room==r) {
			todelete.Add(o);
		} else {
			o->delete_children_in_room(r);
		} /* if */ 
	} /* while */ 

	while(!todelete.EmptyP()) {
		o=todelete.ExtractIni();
		glued_children.DeleteElement(o);
		delete o;
	} /* while */ 
} /* ActiveOBJECT::delete_childrn_in_room */ 


float ActiveOBJECT::absolute_scale()
{
	float base_scale=1.0;

	if (this->parent!=0) base_scale=parent->absolute_scale();

	return base_scale*(float(scale)/100.0F);
} /* absolute_scale */ 


float ActiveOBJECT::absolute_angle()
{
	float base_angle=0;

	if (this->parent!=0) base_angle=parent->absolute_angle();

	if (angle>360) angle-=360;
	if (angle<0) angle+=360;
	return base_angle+float((float(angle)*M_PI)/180.0F);
} /* absolute_angle */ 


int ActiveOBJECT::highest_priority_special(void)
{
	int ps=-1;
	int ss=-1;
	int i;

	for(i=0;i<object->n_s_states;i++) {
		if (g_s_state_executing[i] &&
			object->g_s_state_exclusiveness[i] &&
			object->g_s_state_priority[i]>ps) {
			ss=i;
			ps=object->g_s_state_priority[i];
		} /* if */ 
	} /* for */ 

	return ss;
} /* ActiveOBJECT::highest_priority_special */ 



ActiveMOG2 *ActiveOBJECT::get_game(void)
{
	if (interf!=0) return interf->game;

	if (room!=0 &&
		room->map!=0) return room->map->game;

	return 0;
} /* ActiveOBJECT::get_game */ 


ActiveMAP *ActiveOBJECT::get_map(void)
{
	if (room!=0) return room->map;

	return 0;
} /* ActiveOBJECT::get_map */ 
